/* This file is part of the KDE Project
   Copyright (c) 2018 Oleg Solovyov <mcpain@altlinux.org>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "defaults.h"
#include "kcm_freememorynotifier.h"

#include <KCModule>
#include <KConfigGroup>
#include <KPluginFactory>
#include <KSharedConfig>

#include <QtDBus>

K_PLUGIN_FACTORY_WITH_JSON(KCMFreeMemoryModuleFactory,
                           "kcm_freememorynotifier.json",
                           registerPlugin<KCMFreeMemoryModule>();)

KCMFreeMemoryModuleForm::KCMFreeMemoryModuleForm(QWidget *parent)
    : QWidget(parent)
{
    setupUi(this);
}


KCMFreeMemoryModule::KCMFreeMemoryModule(QWidget *parent, const QVariantList&)
    : KCModule(parent)
{
    m_ui = new KCMFreeMemoryModuleForm(this);

    QGridLayout* layout = new QGridLayout(this);
    layout->addWidget(m_ui, 0, 0);

    load();
}

KCMFreeMemoryModule::~KCMFreeMemoryModule()
{
}

void KCMFreeMemoryModule::load()
{
    KCModule::load();
    KCModule::setButtons(Button::Default | Button::Apply);

    KSharedConfig::Ptr cfg = KSharedConfig::openConfig(QStringLiteral("memorynotifierrc"));
    KConfigGroup grp = cfg->group("General");

    m_ui->kcfg_enableNotification->setChecked(grp.readEntry(CONFIG_ENABLE_NOTIFICATION, DEFAULT_ENABLE_NOTIFICATION));
    m_ui->kcfg_includeSwap->setChecked(grp.readEntry(CONFIG_INCLUDESWAP, DEFAULT_INCLUDESWAP));
    m_ui->kcfg_minimumSpace->setValue(grp.readEntry<int>(CONFIG_THRESHOLD, DEFAULT_THRESHOLD));
    m_ui->kcfg_includeSwap->setEnabled(m_ui->kcfg_enableNotification->isChecked());
    m_ui->kcfg_minimumSpace->setEnabled(m_ui->kcfg_enableNotification->isChecked());


    connect(m_ui->kcfg_enableNotification, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->kcfg_includeSwap, SIGNAL(stateChanged(int)), this, SLOT(changed()));
    connect(m_ui->kcfg_minimumSpace, SIGNAL(valueChanged(int)), this, SLOT(changed()));

    emit changed(false);
}

void KCMFreeMemoryModule::save()
{
    KCModule::save();

    KSharedConfig::Ptr cfg = KSharedConfig::openConfig(QStringLiteral("memorynotifierrc"));
    KConfigGroup grp = cfg->group("General");

    grp.writeEntry(CONFIG_ENABLE_NOTIFICATION, m_ui->kcfg_enableNotification->isChecked());
    grp.writeEntry(CONFIG_INCLUDESWAP, m_ui->kcfg_includeSwap->isChecked());
    grp.writeEntry(CONFIG_THRESHOLD, m_ui->kcfg_minimumSpace->value());
    grp.sync();

    QDBusConnection con = QDBusConnection::sessionBus();
    QDBusConnectionInterface* interface = con.interface();
    QDBusMessage msg = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kded5"),
                                                      QStringLiteral("/modules/freememorynotifier"),
                                                      QStringLiteral("org.kde.FreeMemoryNotifier"),
                                                      QStringLiteral("reload"));

    con.asyncCall(msg);

    emit changed(false);
}

void KCMFreeMemoryModule::defaults()
{
    KCModule::defaults();

    m_ui->kcfg_enableNotification->setChecked(DEFAULT_ENABLE_NOTIFICATION);
    m_ui->kcfg_includeSwap->setChecked(DEFAULT_INCLUDESWAP);
    m_ui->kcfg_minimumSpace->setValue(DEFAULT_THRESHOLD);

    emit changed(true);
}

#include "kcm_freememorynotifier.moc"
